@@ -1,4 +1,3 @@
-{-# LANGUAGE BangPatterns #-}
 {-# LANGUAGE OverloadedStrings #-}
 
 module Network.Socket.ByteString.Lazy.Posix (
@@ -34,7 +33,7 @@ send s lbs = do
   where
     withPokes ss p f = loop ss p 0 0
       where
-        loop (c:cs) q k !niovs
+        loop (c:cs) q k niovs
             | k < maxNumBytes = unsafeUseAsCStringLen c $ \(ptr, len) -> do
                 poke q $ IOVec (castPtr ptr) (fromIntegral len)
                 loop cs
@@ -51,7 +50,10 @@ sendAll
     -> L.ByteString -- ^ Data to send
     -> IO ()
 sendAll _ "" = return ()
-sendAll s bs = do
-    sent <- send s bs
-    waitWhen0 (fromIntegral sent) s
-    when (sent >= 0) $ sendAll s $ L.drop sent bs
+sendAll s bs0 = loop bs0
+  where
+    loop bs = do
+        -- "send" throws an exception.
+        sent <- send s bs
+        waitWhen0 (fromIntegral sent) s
+        when (sent /= L.length bs) $ loop $ L.drop sent bs
